import numpy as np
import numpy.typing as npt
from shapely import Point, creation
from navsim.planning.simulation.planner.pdm_planner.observation.pdm_occupancy_map import (
    PDMDrivableMap,
)
from navsim.planning.simulation.planner.pdm_planner.utils.pdm_path import (
    PDMPath,
)
import math

def calculate_progress_score(progress, threshold, max_value):
    score = np.maximum(0, 1 - (progress - threshold) / (max_value - threshold))
    smaller_than_threshold = progress <= threshold
    score[smaller_than_threshold] = 1.0
    return score

def calculate_non_linear_progress_score(values, threshold, scaling_factor=1.0):
    # 使用非线性函数来计算分数
    scaled_values = np.exp((values - threshold) * scaling_factor) - 1
    scores = 1 / (1 + scaled_values)  # 将分数缩放到0到1之间
    smaller_than_threshold = values <= threshold
    scores[smaller_than_threshold] = 1.0
    return scores

def calculate_angle_score(angle, threshold, max_value):
    score = np.maximum(0, 1 - (angle - threshold) / (max_value - threshold))
    smaller_than_threshold = angle <= threshold
    score[smaller_than_threshold] = 1.0
    return score

def calc_consistency(ego_coords,
            pre_ego_coords,
            BBCoordsIndex,
            num_proposals,
            multi_metrics):
    """
    Calculate the average distance from the vehicle to the centerline.
    """
    if pre_ego_coords is None:
        return [None for _ in range(num_proposals)]
    progress_in_meter = np.zeros(num_proposals, dtype=np.float64)
    angle_consistency = np.zeros(num_proposals, dtype=np.float64)  # piont consistency
    for proposal_idx in range(num_proposals):
        count = 0
        count_angle = 0
        total_distance = 0.0
        normalize_distance = 0.0
        total_angle_diff = 0.0  # Add this for angle difference sum
        count_isok = 0
        for time_idx in range(35):

            vehicle_position = Point(*ego_coords[proposal_idx, time_idx, BBCoordsIndex.CENTER])
            pre_vehicle_position = Point(*pre_ego_coords[0, time_idx + 5, BBCoordsIndex.CENTER])

            #angle consistency
            if time_idx >= 5:
                pre_vehicle_position_prev = Point(*pre_ego_coords[0, time_idx, BBCoordsIndex.CENTER])
                vehicle_position_pre = Point(*ego_coords[proposal_idx, time_idx - 5, BBCoordsIndex.CENTER])
                vector_curr = (
                vehicle_position.x - vehicle_position_pre.x, vehicle_position.y - vehicle_position_pre.y)
                vector_prev = (pre_vehicle_position.x - pre_vehicle_position_prev.x,
                               pre_vehicle_position.y - pre_vehicle_position_prev.y)
                # Calculate angle between vectors
                dot_product = vector_curr[0] * vector_prev[0] + vector_curr[1] * vector_prev[1]
                magnitude_curr = math.sqrt(vector_curr[0] ** 2 + vector_curr[1] ** 2)
                magnitude_prev = math.sqrt(vector_prev[0] ** 2 + vector_prev[1] ** 2)

                if magnitude_curr != 0 and magnitude_prev != 0:
                    cos_theta = dot_product / (magnitude_curr * magnitude_prev)
                    cos_theta = np.clip(cos_theta, -1.0, 1.0)
                    angle_diff = np.arccos(cos_theta)
                    total_angle_diff += angle_diff
                    count_angle += 1

            # if time_idx >= 1:
                # pre_vehicle_position_prev = Point(*pre_ego_coords[0, time_idx + 4, BBCoordsIndex.CENTER])
                # vehicle_position_pre = Point(*ego_coords[proposal_idx, time_idx - 1, BBCoordsIndex.CENTER])
                # normalize_distance += vehicle_position.distance(vehicle_position_pre)
                # Calculate the angle consistency (direction difference)
                # vector_curr = (
                # vehicle_position.x - vehicle_position_pre.x, vehicle_position.y - vehicle_position_pre.y)
                # vector_prev = (pre_vehicle_position.x - pre_vehicle_position_prev.x,
                #                pre_vehicle_position.y - pre_vehicle_position_prev.y)
                # Calculate angle between vectors
                # dot_product = vector_curr[0] * vector_prev[0] + vector_curr[1] * vector_prev[1]
                # magnitude_curr = math.sqrt(vector_curr[0] ** 2 + vector_curr[1] ** 2)
                # magnitude_prev = math.sqrt(vector_prev[0] ** 2 + vector_prev[1] ** 2)

                # if magnitude_curr != 0 and magnitude_prev != 0:
                #     cos_theta = dot_product / (magnitude_curr * magnitude_prev)
                #     cos_theta = np.clip(cos_theta, -1.0, 1.0)  # To avoid numerical errors
                #     angle_diff = np.arccos(cos_theta)
                #     total_angle_diff += angle_diff
                #     count_angle += 1
            # Calculate the distance between two traj
            distance = vehicle_position.distance(pre_vehicle_position)
            total_distance += distance
            count += 1
        # 归一化后的距离，越小越好

        progress_in_meter[proposal_idx] = total_distance / count
        angle_consistency[proposal_idx] = total_angle_diff / count_angle
        # Normalize position and direction consistency to [0, 1] scale
    # for i in range(100000):
    #     print("progress", progress_in_meter.mean(), progress_in_meter.min(), progress_in_meter.max())
    #     print("angle", angle_consistency.mean(), angle_consistency.min(), angle_consistency.max())



    # return progress_in_meter
    normalized_progress = np.ones_like(progress_in_meter)
    normalized_angle = np.ones_like(progress_in_meter)
    # # 设置一个阈值，如果偏移量小于这个阈值，就设置为满分
    # # good_score = multiplicate_metric_scores != 0.0

    # threhold_progress = 0.05
    # max_value_progress = np.max(progress_in_meter)
    # normalized_progress = calculate_non_linear_progress_score(progress_in_meter, threhold_progress, 5)
    smaller_than_dis_mask = progress_in_meter <= 0.5 # (4096,1)
    bigger_than_dis_mask = np.logical_not(smaller_than_dis_mask)
    normalized_progress[bigger_than_dis_mask] = 0.5 / progress_in_meter[bigger_than_dis_mask]
    normalized_progress[smaller_than_dis_mask] = 1.0
    threhold_angle = 0.1
    # max_value_angle = np.max(angle_consistency)
    normalized_angle = calculate_non_linear_progress_score(angle_consistency, threhold_angle, 5)
    # smaller_than_angle_mask = angle_consistency <= 10 * 3.1415 / 180.0 # (4096,1)
    # bigger_than_angle_mask = np.logical_not(smaller_than_angle_mask)
    # normalized_angle[bigger_than_angle_mask] = (10 * 3.1415 / 180.0) / angle_consistency[bigger_than_angle_mask]
    # normalized_angle[smaller_than_angle_mask] = 1.0

    # progress_in_meter_norm = (progress_in_meter - progress_in_meter.min()) / (
    #             progress_in_meter.max() - progress_in_meter.min())
    # angle_consistency_norm = (angle_consistency - angle_consistency.min()) / (
    #             angle_consistency.max() - angle_consistency.min())
    # Combine the normalized metrics using weights

    for i in range(10):
        print("progress", normalized_progress.mean())
        print("angle", normalized_angle.mean())
    w1 = 0.5   # Weight for Position Consistency
    w2 = 0.5   # Weight for Direction Consistency
    overall_consistency = normalized_progress * normalized_angle
    return overall_consistency
